#ifndef _AGPDRV_H
#define _AGPDRV_H

/* This file is for use by the agpgart DRIVER.
 * User-level code should include "agpgart.h" instead
 * Philip Brown, phil@bolthole.com
 * @(#) agpdrv.h 1.9@(#)
 */


/* Entry for our "key" table, that links "keys" to dma memory
 * we have allocated for use with the GATT table.
 * The key value is the index in the table.
 * Use add_keyent()/del_keyent()
 */
struct keytable_ent {
	ddi_dma_handle_t dma_handle;
	ddi_acc_handle_t dma_acc;
	ddi_dma_cookie_t dma_cookie;
	int num_dma_cookies;
	ddi_umem_cookie_t umem_cookie;
	paddr_t phys_addr;
	caddr_t kern_addr;
	uint32_t agp_offset; /* place in the agp aperture this is mapped to */
	size_t length; /* "length" is user-requested size */
	int am_bound;
};

/* This gets filled out on a per-instance basis by agpgart_attach */
struct agpgart_softstate {
	dev_info_t *dip; /* you'll probably want this saved! */
	kmutex_t instance_mutex;
	kmutex_t intr_mutex;
	ddi_iblock_cookie_t intr_cookie;
	int am_open, am_enabled;
	/* am_open just says user has us open.
	 * am_enabled says that AGPIOC_SETUP has been called, and
	 * we have enabled the AGP mappings, GART table, etc.
	 * am_enabled is currently set in _SETUP, but
	 * is unset in _RELEASE
	 */

	ddi_acc_handle_t pcihandle;	/* for access to pci config regs */
	
	int aperture_size;	/* current setting of aperture in BYTES */
	uint32_t agp_mode;	/* straight from CONF_something */
	ddi_device_acc_attr_t *dev_dma_attr; /* */
	struct keytable_ent * keytable; /* see AGP_MAXKEYS */

	caddr_t GATT;		/* kernel-writable addr for GATT table */
	uint32_t GATTphysaddr;	/* AGP hardware is only 32bit */
	size_t GATTlength;
	ddi_dma_handle_t GATThandle;
	ddi_acc_handle_t GATTacc;
};

#define AGP_PAGE_SIZE	4096	/* 4k is page size for AGP hardware */



/* Im just guessing on some of these values like maxxfer */
static ddi_dma_attr_t dev_dma_attr = {
	DMA_ATTR_V0,
	(unsigned long long)0,		/* dma_attr_addr_lo */
	(unsigned long long)0xffffffff,	/* dma_attr_addr_hi */
	(unsigned long long)0xffffff,	/* dma_attr_count_max */
	(unsigned long long)AGP_PAGE_SIZE,	/* dma_attr_align */
	1 ,				/* dma_attr_burstsizes */
	1,				/* dma_attr_minxfer */
	(unsigned long long)0x3ffffff,	/* dma_attr_maxxfer */
	(unsigned long long)0xffffffff,	/* dma_attr_seg */
	1,				/* dma_attr_sgllen */
	AGP_PAGE_SIZE,			/* dma_attr_granular */
        0				/* dma_attr_flags */
};



/* These numbers match the linux source I'm copying the API from */
#define VERSION_MAJOR 0
#define VERSION_MINOR 99

/******************************************************************
 * "backend" function prototypes in the main line, that hardware *
 * specific stuff can call:
 ******************************************************************/
/* This really belongs in the intel section*/
size_t agp_apsize_to_bytes(uint8_t apsize);

int agp_alloc_mem(struct agpgart_softstate *softstate,size_t length);
int agp_dealloc_mem(struct agpgart_softstate *softstate,int key);

void agp_del_allkeys(struct agpgart_softstate *softstate);
struct keytable_ent* agp_get_keyent(struct agpgart_softstate *,int );
/* Count number of pages allocated, in key tables */
int agp_count_keypages(struct agpgart_softstate *softstate);

#endif /*  _AGPDRV_H */
